package de.intevation.flys.artifacts.math.fitting;

public class LogLinear
extends      Function
{
    public static final Derivative DERIVATIVE =
        new Derivative("W'(Q) = a*m/(m*Q + b)") {

        @Override
        public de.intevation.flys.artifacts.math.Function
            instantiate(double [] parameters)
        {
            final double a = parameters[0];
            final double m = parameters[1];
            final double b = parameters[2];

            return new de.intevation.flys.artifacts.math.Function() {
                @Override
                public double value(double Q) {
                    return a*m/(m*Q + b);
                }
            };
        }
    };

    public LogLinear() {
        super(
            "log-linear",
            "W(Q) = a*ln(m*Q + b)",
            new String [] { "a", "m", "b" });
    }

    @Override
    public double value(double x, double [] parameters) {
        return parameters[0]*Math.log(parameters[1]*x + parameters[2]);
    }

    @Override
    public double [] gradient(double x, double [] parameters) {
        double l = parameters[1]*x + parameters[2];
        return new double [] {
            Math.log(l),
            parameters[0]*x/l,
            parameters[0]/l
        };
    }

    @Override
    public Derivative getDerivative() {
        return DERIVATIVE;
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
